local encounters = HudMap:GetModule("Encounters")
local HudMap = _G.HudMap
local UnitName, UnitIsDead = _G.UnitName, _G.UnitIsDead
local parent = HudMap
local L = LibStub("AceLocale-3.0"):GetLocale("HudMap")
local SN = parent.SN

local COLLECT_DEBUG_DATA = false

local function register(e)	
	encounters:RegisterEncounterMarker(e)
	return e
end

local free = parent.free

local halion = {
	name = L["Halion"],	
	startEncounterIDs = 39863,
	endEncounterIDs = 39863,
	options = {
		fieryCombustion = SN[74562],
		soulConsumption = SN[74792],
		-- meteorStrike = SN[75877],
	},
	defaults = {
		fieryCombustionColor = {r = 1, g = 0.35, b = 0, a = 0.4},
		soulConsumptionColor = {r = 0, g = 0.35, b = 0.5, a = 0.4},
		meteorStrikeColor = {r = 1, g = 0.35, b = 0, a = 0.4}
	},
	pew_pew_lazers_center = { 370, 229.3 },
	radius = 95 / 2,
	MARK_OF_CONSUMPTION = 74567,
	MARK_OF_COMBUSTION = 74795,
	marks = {},
	markMarkers = {},
	setupDebug = function(self)
		if COLLECT_DEBUG_DATA then
			HudMapDebug = HudMapDebug or {}
			HudMapDebug["cutters"] = HudMapDebug["cutters"] or {}
			return HudMapDebug["cutters"]
		else
			self.debugTemp = self.debugTemp or {}
			return self.debugTemp
		end
	end,
	doMark = function(self, spellID, destName, isLong)
		self.markMarkers[destName] = free(self.markMarkers[destName], self, "mark" .. spellID .. destName)
		local stacks = (self.marks[destName] or 1)
		if stacks < 1 then stacks = 1 end
		local size = stacks * 4
		local timer = isLong and 40 or 2
		local r, g, b, a
		if spellID == 74567 and self:Option("fieryCombustionEnabled") then
			r, g, b, a = self:Option("fieryCombustionColor")
		elseif spellID == 74795 and self:Option("soulConsumptionEnabled") then
			r, g, b, a = self:Option("soulConsumptionColor")
		end	
		if r then
			local t
			if isLong then
				local x, y = HudMap:GetUnitPosition(destName)
				t = register(HudMap:PlaceRangeMarker("timer", x, y, size, timer, r, g, b, a))
			else
				t = HudMap:PlaceRangeMarkerOnPartyMember("timer", destName, size, timer, r, g, b, a):Identify(self, "mark" .. spellID .. destName)
				self.markMarkers[destName] = t
			end
			register(t):Rotate(360, timer):Appear():RegisterForAlerts():SetLabel(destName):Broadcast()
		end
	end,
	SPELL_AURA_APPLIED = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		if spellID == 71289 and self:Option("meteorStrikeEnabled") then
			-- local r, g, b, a = self:Option("meteorStrikeColor")
			-- self.markMarkers[destName] = register(HudMap:PlaceRangeMarkerOnPartyMember("timer", destName, 8, 12, r, g, b, a):Rotate(360, 12):Appear():RegisterForAlerts():SetLabel(destName))
			
		elseif spellID == self.MARK_OF_CONSUMPTION or spellID == self.MARK_OF_COMBUSTION then
			self.marks[destName] = 1
			self:doMark(spellID, destName)
		end
	end,
	SPELL_AURA_APPLIED_DOSE = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		-- Consumption, Combustion
		if spellID == self.MARK_OF_CONSUMPTION or spellID == self.MARK_OF_COMBUSTION then
			self.marks[destName] = self.marks[destName] + 1
			self:doMark(spellID, destName)
		end
	end,
	SPELL_AURA_REMOVED = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		if spellID == self.MARK_OF_CONSUMPTION or spellID == self.MARK_OF_COMBUSTION then
			self:doMark(spellID, destName, true)
		end
	end,
	SPELL_DAMAGE = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		-- Measure unit position and time offset if they take Twilight Cutter damage
		-- This will be used to derive beam positions, which can be reversed into angular velocities and starting positions
		if spellID == 77844 then
			local x, y = HudMap:GetUnitPosition(destName)
			local offset = GetTime() - (self.phaseTwoStartTime or 0)
			local t = self:setupDebug()
			tinsert(t, {
				event = "damage",
				timecode = GetTime(),
				offset = offset,
				player = destName,
				x = x,
				y = y
			})
		end	
	end,
	CHAT_MSG_RAID_BOSS_EMOTE = function(self, msg)
		if msg == L["The orbiting spheres pulse with dark energy!"] then
			local offset = GetTime() - (self.phaseTwoStartTime or 0)
			local t = self:setupDebug()
			tinsert(t, {
				event = "warning",
				timecode = GetTime(),
				offset = offset
			})
		end
	end,
	CHAT_MSG_MONSTER_YELL = function(self, msg)
		if msg == L["You will find only suffering within the realm of twilight! Enter if you dare!"] then
			self.phaseTwoStartTime = GetTime()
			local t = self:setupDebug()
			tinsert(t, {
				event = "phase2",
				timecode = GetTime()
			})
		end
	end
}

encounters:RegisterModule(L["Ruby Sanctum"], halion)